;##################################################################
; ACM CODEC Enumeration
;
; From the MSDN example: Using CODECs to Compress Wave Audio
; http://msdn.microsoft.com/archive/default.asp?url=/archive/en-us/dnarmulmed/html/msdn_codec.asp
; Translated to GoAsm by Edgar Hansen 2007
;##################################################################

#include "EnumCodecs.Inc"
#include "msacm.inc"

#DYNAMICLINKFILE Msacm32.dll

DATA SECTION
	hInstance	DD		?
	hListbox	DD		?
	buffer		DB		1024 DUP (?)

CODE SECTION

Start:
	invoke GetModuleHandle, 0
	mov [hInstance],eax
	invoke InitCommonControls
	invoke DialogBoxParam,[hInstance],IDD_DLG1,0,ADDR DlgProc,0
	invoke ExitProcess,0

DlgProc FRAME hwnd,uMsg,wParam,lParam

	mov eax,[uMsg]
	cmp eax,WM_INITDIALOG
	jne >.WM_CLOSE
		invoke GetDlgItem,[hwnd],1001
		mov [hListbox],eax
		invoke GetACMInfo
		invoke LBAddString, "Enabled drivers :"
		invoke LBAddString, ""
		invoke acmDriverEnum, offset DriverEnumProc, 0, 0
		
		jmp >.EXIT

	.WM_CLOSE
	cmp eax,WM_CLOSE
	jne >.DEFPROC
		INVOKE EndDialog,[hwnd],0

	.DEFPROC
		mov eax,FALSE
		ret

	.EXIT

	mov eax, TRUE
	ret
ENDF

LBAddString FRAME string
	invoke SendMessage,[hListbox],LB_ADDSTRING,0,[string]
	RET
ENDF

GetACMInfo FRAME
	LOCAL dwCodecs	:D
	LOCAL dwACMVer	:D
	LOCAL mmr		:D

	mov D[dwCodecs],0

	invoke acmGetVersion
	mov [dwACMVer], eax
	
	mov eax,[dwACMVer]
	and eax,0FFFF0000h
	shr eax,24
	
	mov ecx,[dwACMVer]
	and ecx,0FFFF0000h
	shr ecx, 16
	and ecx,0FFh
	
	mov edx,[dwACMVer]
	and edx,0FFFFh
	
	test edx,edx
	jnz >
		invoke lstrcpy,offset buffer,"Retail"
		jmp >.SHOWVERSION
	:
	invoke wsprintf,offset buffer,"ACM version %u.%.02u build %u",eax,ecx,edx
	add esp,20
	.SHOWVERSION

    invoke LBAddString, offset buffer
	
	invoke acmMetrics,0, ACM_METRIC_COUNT_CODECS, offset dwCodecs
	mov [mmr], eax
	
	invoke wsprintf,offset buffer,"Codecs installed : %u",[dwCodecs]
	add esp,12
	invoke LBAddString, offset buffer
	invoke LBAddString, ""
	RET
ENDF

DriverEnumProc FRAME hadid, dwInstance, fdwSupport
	LOCAL DrvDet			:ACMDRIVERDETAILS
	LOCAL had				:D
	LOCAL dwSize			:D
	LOCAL pwf				:D
	LOCAL fd				:ACMFORMATDETAILS
	LOCAL pwfx				:D

	invoke wsprintf,offset buffer,"id: %8.8lxH",[hadid]
	add esp,12

	invoke LBAddString, offset buffer

	invoke LBAddString, "Supports:"

	test D[fdwSupport],ACMDRIVERDETAILS_SUPPORTF_ASYNC
    jz>
		invoke LBAddString, "   async conversions"
	:
    
	test D[fdwSupport],ACMDRIVERDETAILS_SUPPORTF_CODEC
    jz>
		invoke LBAddString, "   different format conversions"
	:

	test D[fdwSupport],ACMDRIVERDETAILS_SUPPORTF_CONVERTER
    jz>
		invoke LBAddString, "   same format conversions"
	:

	test D[fdwSupport],ACMDRIVERDETAILS_SUPPORTF_FILTER
    jz>
		invoke LBAddString, "   filtering"
	:

	invoke LBAddString, ""
	invoke LBAddString, "Driver details"

    // Get some details.
	mov D[DrvDet.cbStruct], sizeof ACMDRIVERDETAILS

	invoke acmDriverDetails, [hadid], offset DrvDet, 0
	test eax,eax
	jz >
		invoke LBAddString, "Error extracting ACMDRIVERDETAILS"
		mov eax, TRUE
		ret
	:
		invoke wsprintf,offset buffer,"   Short name: %s",offset DrvDet.szShortName
		add esp,12
		invoke LBAddString, offset buffer
		
		invoke wsprintf,offset buffer,"   Long name: %s",offset DrvDet.szLongName
		add esp,12
		invoke LBAddString, offset buffer
		
		invoke wsprintf,offset buffer,"   Copyright: %s",offset DrvDet.szCopyright
		add esp,12
		invoke LBAddString, offset buffer
		
		invoke wsprintf,offset buffer,"   Licensing: %s",offset DrvDet.szLicensing
		add esp,12
		invoke LBAddString, offset buffer
		
		invoke wsprintf,offset buffer,"   Features: %s",offset DrvDet.szFeatures
		add esp,12
		invoke LBAddString, offset buffer
		
		invoke wsprintf,offset buffer,"   Supports %u formats",[DrvDet.cFormatTags]
		add esp,12
		invoke LBAddString, offset buffer
		
		invoke wsprintf,offset buffer,"   Supports %u filter formats",[DrvDet.cFilterTags]
		add esp,12
		invoke LBAddString, offset buffer

	mov D[had], 0
	invoke acmDriverOpen, offset had, [hadid], 0
	test eax,eax
	jz >
		invoke LBAddString, "Error extracting ACM Format details"
		mov eax, TRUE
		ret
	:
		invoke RtlZeroMemory,offset fd,SIZEOF ACMFORMATDETAILS
		invoke LBAddString, ""
		invoke LBAddString, "ACM Format details"
		mov D[dwSize], 0
		invoke acmMetrics, [had], ACM_METRIC_MAX_SIZE_FORMAT, offset dwSize
		cmp D[dwSize],SIZEOF WAVEFORMATEX
		jge >
			mov D[dwSize], SIZEOF WAVEFORMATEX
		:
		invoke GlobalAlloc,GMEM_FIXED,[dwSize]
		mov [pwfx],eax
		mov ecx,[dwSize]
		mov D[eax+WAVEFORMATEX.cbSize], ecx
		mov D[fd.cbwfx], ecx
		mov D[eax+WAVEFORMATEX.wFormatTag], WAVE_FORMAT_UNKNOWN
		mov D[fd.cbStruct], sizeof ACMFORMATDETAILS
		mov eax,[pwfx]
		mov D[fd.pwfx], eax
		mov D[fd.dwFormatTag], WAVE_FORMAT_UNKNOWN
		invoke acmFormatEnum, [had], offset fd, offset FormatEnumProc, 0, 0

		invoke acmDriverClose, [had], 0
		invoke GlobalFree,[pwfx]

	invoke LBAddString, ""
	invoke LBAddString, "**********************************************************"
	invoke LBAddString, ""

	mov eax, TRUE
	ret
ENDF

FormatEnumProc FRAME hadid, pafd, dwInstance, fdwSupport

	mov eax,[pafd]
	add eax,ACMFORMATDETAILS.dwFormatTag

	mov ecx,[pafd]
	add ecx,ACMFORMATDETAILS.szFormat

	invoke wsprintf,offset buffer,"    %4.4lXh - %s", [eax], ecx
	add esp,16
	invoke LBAddString, offset buffer
	mov eax, TRUE; // Continue enumerating.
	RET
ENDF

